/*********************************************************************
 * Filename:      snmpd-sys.c
 * Version:       1.0.2 Final Release
 * Description:   Sublevel program to provide information to snmpd
 * Date:          Fri 26th April 1999 
 * Author:        Sveinar Rasmussen (sveinar@rasmussen.org)
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <utmp.h>
#include <sys/types.h>
#include <time.h>
#ifdef OS_UNIX
#include <sys/timeb.h>
#endif
#ifdef OS_IRIX 
#include <sys/statfs.h> 
#endif 
#include <ctype.h>     /* FSTAB */
#include <sys/vfs.h>
#ifndef OS_SUNOS
#include <mntent.h>    /* END FSTAB */
#endif
#include <netdb.h>     /* NET */
#ifdef SPEC_HPUX 
#include <sys/pstat.h> 
#endif 
#include <malloc.h>
#ifdef OS_SUNOS
#include <sys/mnttab.h>
#include <sys/statfs.h>
#include <netdb.h>
#endif

#define OP_UPTIME   0
#define OP_FSTAB    1
#define OP_NET      2
#define OP_CURRTIME 3
#define OP_HPMEM    4
#define OP_MAXPROCS 5
#define OP_GETPROCS 6

/*==================================================================+
** Uptime from utmp
**=================================================================*/
void uptime()
{
  struct utmp *u;
  long cur_time;
#ifdef OS_UNIX
  struct timeb t; ftime(&t); cur_time = t.time;
#else
  cur_time = time(0);
#endif

  if (NULL==(u = getutent())) {
    printf("-1");
    exit(1);
  } 
  printf("%ld\n", (cur_time - u->ut_time)*100);
}

/*==================================================================+
** File systems
**=================================================================*/
#define MNTTAB_FILE    "/etc/fstab"
#define FS_REACHABLE   "yes"
#define FS_UNREACHABLE "no"
/* Check white space */
static char* check_wss (char * word) 
{
  char * p;
  if (word == NULL || *word == '\0')
    return "-";
  for (p = word; *p != '\0'; p++)
    if (isspace ((int) *p))
      *p = '-';
  return word;
}

void fstab() 
{
  FILE * mntTabFp;
#ifdef OS_SUNOS
  struct mnttab * mntTabEnt;
  struct statvfs sb;
#else
  struct mntent * mntTabEnt;  /* mount table entry */
  struct statfs sb;  /* super block data */
#endif
  unsigned long unixFSID;
  int unixFSReachable;

  if ((mntTabFp = fopen (MNTTAB_FILE, "r")) == NULL)
  {
    fprintf (stderr, "Error in fopen(): cannot open %s for reading\n",
      MNTTAB_FILE);
    perror ("");
    exit (1);
  }

  
#ifdef OS_SUNOS
  for (unixFSID = 1; getmntent(mntTabFp, mntTabEnt) == 0; unixFSID++) {
#else 
  for (unixFSID = 1; (mntTabEnt = getmntent (mntTabFp)) != NULL; unixFSID++)   {
#endif
#ifdef OS_IRIX  
    if (statfs (mntTabEnt->mnt_dir, &sb, sizeof(sb), 0) == 0) unixFSReachable = 1; 
#else 
#ifdef OS_SUNOS
    if (statvfs (mntTabEnt->mnt_mountp, &sb) == 0) unixFSReachable = 1;
#else
    if (statfs (mntTabEnt->mnt_dir, &sb) == 0) unixFSReachable = 1;
#endif
#endif
    else {
      unixFSReachable = 0;
      sb.f_bsize = -1L;
      sb.f_blocks = -1L;
      sb.f_bfree = -1L;
      sb.f_files = -1L;
      sb.f_ffree = -1L;
    }
    printf ("%lu ", unixFSID);   /* print ID */
    /* unixFSName = path of the device to be mounted */
#ifdef OS_SUNOS
    printf ("%s ", check_wss(mntTabEnt->mnt_special));
    printf ("%s ", check_wss(mntTabEnt->mnt_mountp));
    printf ("%s ", check_wss(mntTabEnt->mnt_fstype));
#else
    printf ("%s ", check_wss(mntTabEnt->mnt_fsname));
    printf ("%s ", check_wss(mntTabEnt->mnt_dir));
    printf ("%s ", check_wss(mntTabEnt->mnt_type));
#endif
    printf ("%d ", (int)sb.f_bsize);
    /* unixFSBlockCount = total # of blocks in units of unixFSBlockSize */
    printf ("%d ", (int)sb.f_blocks);
    /* unixFSBlocksFree = total # of free blocks (in units of unixFSBlockSize) */
    printf ("%d\n", (int)sb.f_bfree);
  }
}

/*==================================================================+
** Networks
**=================================================================*/
void net()
{
  struct netent *n;
  if (NULL==(n = getnetent())) {
    printf("-1");
    exit(1);
  } 
  printf("%ld ", n->n_net);
  while (NULL!=(n = getnetent()))  printf("%ld ", n->n_net);
  printf("\n");
}

/*==================================================================+
** Current time
**=================================================================*/
void currTime()
{
  long cur_time;
  cur_time = time(0);
  printf("%ld\n", cur_time);
}

/*==================================================================+
** Get HP Total memory
**=================================================================*/
void getHPmem()
{
#ifdef SPEC_HPUX
  struct pst_static buf;
  pstat_getstatic( &buf, sizeof(buf), 1, 0);
  printf ("%ld\n", (buf.page_size/1024)*buf.physical_memory);
#endif
}

/*==================================================================+
** Get maximum number of procs :)
**=================================================================*/
void maxProcs()
{
#ifdef SPEC_HPUX
  struct pst_static buf;
  pstat_getstatic( &buf, sizeof(buf), 1, 0);
  printf ("%ld\n", buf.max_proc);
#endif
}

/*==================================================================+
** Get processes (HPUX only)
**=================================================================*/
void getProcs()
{
#ifdef SPEC_HPUX
  struct pst_status *proc;
  struct pst_static buf;
  int i, j, mem, time;
  char state;
  pstat_getstatic( &buf, sizeof(buf), 1, 0);
  proc = (struct pst_status*) malloc( sizeof(struct pst_status)*buf.max_proc );
  printf ("%ld\n", buf.max_proc);
  j = pstat_getproc (proc, sizeof(struct pst_status), buf.max_proc, 0);
  pstat_getstatic( &buf, sizeof(buf), 1, 0);
  printf ("PROCS:\n");
  for (i=0; i<j; i++) {
    if (proc[i].pst_stat==PS_RUN) state = 'R';
    else if (proc[i].pst_stat==PS_ZOMBIE) state = 'N';
    else if (proc[i].pst_stat==PS_OTHER) state = 'N';
    else state = 'S';
    mem = (buf.page_size/1024)*(proc[i].pst_vtsize + proc[i].pst_vdsize +\
          proc[i].pst_vssize);
    time = (proc[i].pst_utime+proc[i].pst_stime)/6;
    printf(" - %c - %ld - - - - - %d - - %d %s \n", state,
	   proc[i].pst_pid, mem, time, proc[i].pst_cmd );
  }
  free (proc);
#endif
}

/*==================================================================+
** Main dispatcher
**=================================================================*/
int main(int argc, char **argv)
{
  if (argc==2) {
    switch (atoi(argv[1])) {
    case OP_UPTIME:   { uptime();   exit(0);}
    case OP_FSTAB:    { fstab();    exit(0);}
    case OP_NET:      { net();      exit(0);}
    case OP_CURRTIME: { currTime(); exit(0);}
    case OP_HPMEM:    { getHPmem(); exit(0);}
    case OP_MAXPROCS: { maxProcs(); exit(0);}
    case OP_GETPROCS: { getProcs(); exit(0);}
    default: {}  
    }
  }  
  printf ("Internal sub program for snmpd.\n");
  return 0;
}
